"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const mergeOptions_1 = __importDefault(require("./mergeOptions"));
/**
 * @hidden
 *
 * Task Runner runs only the
 * controlled number of tasks at any given
 * instance of time.
 *
 * @param worker function to be called for executing the task
 * @param options
 *
 * @returns Task adder function
 */
class TaskQueue {
    constructor(worker, options = {}) {
        this.worker = worker;
        this.tasks = [];
        this.running = 0;
        this.options = mergeOptions_1.default(options, {
            maxRunners: 10,
        });
    }
    add(...args) {
        return new Promise((resolve) => {
            // add task to DLL
            this.tasks.push({
                args,
                resolve,
            });
            // inform runner about the new task
            this.runner();
        });
    }
    /**
     * This is responsible for running the tasks
     * in parallel and to maintain only the
     * limited number of concurrent tasks
     */
    runner() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.running < this.options.maxRunners) {
                this.running++;
                const task = this.tasks.shift();
                if (task) {
                    const { args, resolve } = task;
                    try {
                        const data = yield this.worker(...args);
                        resolve([null, data]);
                    }
                    catch (error) {
                        resolve([error]);
                    }
                    this.running--;
                    // assuming a new task might be present,
                    // we're calling the runner again.
                    this.runner();
                }
                else {
                    // since no task has been found,
                    // we'll exit the runner process
                    this.running--;
                }
            }
        });
    }
}
exports.default = TaskQueue;
// export default function TaskQueue(worker: (...args: any[]) => any, { maxRunners = 10 }: IOptions = {}): IRunTask {
//   const tasks: ITask[] = [];
//   // number of currently running tasks
//   const running: number = 0;
//   runner();
//   return {
//     add(...args: any[]) {
//       return new Promise((resolve) => {
//         // add task to DLL
//         tasks.push({
//           args,
//           resolve,
//         });
//         // inform runner about the new task
//         runner();
//       });
//     },
//   };
// }
//# sourceMappingURL=taskQueue.js.map