"use strict";
/* tslint:disable no-unused-expression */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const mergeOptions_1 = __importDefault(require("../../src/lib/mergeOptions"));
describe('#mergeOptions', () => {
    it('should be able to merge two object', () => {
        const givenOpts = {
            foo: 1,
            bar: 2,
        };
        const defaultOpts = {
            baz: 3,
        };
        chai_1.expect(mergeOptions_1.default(givenOpts, defaultOpts)).to.have.keys(['foo', 'bar', 'baz']);
    });
    it('should prioritize givenOpts over defaultOpts', () => {
        const givenOpts = {
            foo: 1,
            bar: 2,
        };
        const defaultOpts = {
            foo: 3,
        };
        chai_1.expect(mergeOptions_1.default(givenOpts, defaultOpts)).to.have.keys(['foo', 'bar']);
        chai_1.expect(mergeOptions_1.default(givenOpts, defaultOpts).foo).to.eql(1);
    });
    it('should be able to merge nested objects', () => {
        const givenOpts = {
            foo: 1,
            bar: 2,
            baz: {
                foo: 3,
            },
        };
        const defaultOpts = {
            baz: {
                foo: 4,
                daz: 5,
            },
        };
        chai_1.expect(mergeOptions_1.default(givenOpts, defaultOpts)).to.have.keys(['foo', 'bar', 'baz']);
        chai_1.expect(mergeOptions_1.default(givenOpts, defaultOpts).baz).to.have.keys(['foo', 'daz']);
        chai_1.expect(mergeOptions_1.default(givenOpts, defaultOpts).baz.foo).to.eql(3);
        chai_1.expect(mergeOptions_1.default(givenOpts, defaultOpts).baz.daz).to.eql(5);
    });
});
//# sourceMappingURL=mergeOptions.spec.js.map