"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("mocha");
const chai_1 = require("chai");
const delay_1 = __importDefault(require("delay"));
const pipe_1 = require("../../src/lib/pipe");
const getCtx = () => ({
    data: null,
    pubEvent: 'foo/bar',
    subEvents: ['foo/bar'],
});
// describe('#compose', () => {
//   it('should compose all the given handlers', () => {
//     let processedId = 0;
//     compose(() => {
//       expect(processedId).to.be.eql(2);
//     }, () => {
//       expect(processedId).to.be.eql(1);
//       processedId = 2;
//     }, () => {
//       expect(processedId).to.be.eql(0);
//       processedId = 1;
//     })(getCtx());
//   });
//   it('should compose all async handlers', async () => {
//     let processedId = 0;
//     compose(() => {
//       expect(processedId).to.be.eql(2);
//     }, async () => {
//       expect(processedId).to.be.eql(1);
//       await new Promise(res => setTimeout(() => {
//         processedId = 2;
//         res();
//       }, 10));
//     }, async () => {
//       expect(processedId).to.be.eql(0);
//       await new Promise(res => setTimeout(() => {
//         processedId = 1;
//         res();
//       }, 10));
//     })(getCtx());
//   });
// });
describe('#pipe', () => {
    it('should pipe all the handlers', () => {
        let processedId = 0;
        pipe_1.pipe(() => {
            chai_1.expect(processedId).to.be.eql(0);
            processedId = 1;
            return null;
        }, () => {
            chai_1.expect(processedId).to.be.eql(1);
            processedId = 2;
            return null;
        }, () => {
            chai_1.expect(processedId).to.be.eql(2);
            return null;
        })(getCtx());
    });
    it('should pipe all async handlers', () => {
        let processedId = 0;
        pipe_1.pipe(() => {
            chai_1.expect(processedId).to.be.eql(0);
            return new Promise(res => setTimeout(() => {
                processedId = 1;
                res(null);
            }, 10));
        }, () => {
            chai_1.expect(processedId).to.be.eql(1);
            return new Promise(res => setTimeout(() => {
                processedId = 2;
                res(null);
            }, 10));
        }, () => {
            chai_1.expect(processedId).to.be.eql(2);
        })(getCtx());
    });
    it('should pass on the results returned by intermediate handler to next handler', () => {
        pipe_1.pipe(({ data }) => {
            chai_1.expect(data).to.be.null;
            return 1;
        }, ({ data }) => {
            chai_1.expect(data).to.be.eql(1);
            return 2;
        }, ({ data }) => {
            chai_1.expect(data).to.be.eql(2);
        })(getCtx());
    });
    it('should pass the second or the further args as is without any modifications to all the handlers', () => {
        pipe_1.pipe(({ data }, ev) => __awaiter(void 0, void 0, void 0, function* () {
            chai_1.expect(ev).to.be.eql('test');
            chai_1.expect(data).to.be.null;
            // to simulate async nature
            yield delay_1.default(10);
            return 1;
        }), ({ data }, ev) => {
            chai_1.expect(ev).to.be.eql('test');
            chai_1.expect(data).to.be.eql(1);
            return 2;
        }, ({ data }, ev) => {
            chai_1.expect(ev).to.be.eql('test');
            chai_1.expect(data).to.be.eql(2);
        })(getCtx(), 'test');
    });
    it('should be able to stop the pipeline flow by returning nothing | undefined', () => __awaiter(void 0, void 0, void 0, function* () {
        let calls = 0;
        const data = yield pipe_1.pipe(() => {
            calls++;
            return;
        }, () => {
            calls++;
        })(getCtx());
        chai_1.expect(calls).to.be.eql(1);
        chai_1.expect(data).to.be.eql(undefined);
        calls = 0;
        yield pipe_1.pipe(() => {
            calls++;
        }, () => {
            calls++;
        })(getCtx());
        chai_1.expect(calls).to.be.eql(1);
        chai_1.expect(data).to.be.eql(undefined);
    }));
});
//# sourceMappingURL=pipe.spec.js.map