"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const supervisedEmitter_1 = __importDefault(require("../src/supervisedEmitter"));
const { patternHandler } = supervisedEmitter_1.default;
describe('#patternHandler', () => {
    it('should return a middleware which listens only to the given pattern', () => __awaiter(void 0, void 0, void 0, function* () {
        let calls = 0;
        const SE = new supervisedEmitter_1.default([
            patternHandler('hello/**', () => {
                calls++;
                return null;
            }),
        ]);
        yield SE.publish('hello/world', 'test');
        yield SE.publish('fun/world', 'test');
        chai_1.expect(calls).to.be.eql(1);
    }));
    it('should return a middleware which listens only to the given topic', () => __awaiter(void 0, void 0, void 0, function* () {
        let calls = 0;
        const SE = new supervisedEmitter_1.default([
            patternHandler('hello/world', () => {
                calls++;
                return null;
            }),
        ]);
        yield SE.publish('hello/world', 'test');
        yield SE.publish('fun/world', 'test');
        chai_1.expect(calls).to.be.eql(1);
    }));
    it('should pass through the data if the pattern doesnt match with the pubEvent', () => __awaiter(void 0, void 0, void 0, function* () {
        const calls = [0, 0];
        const SE = new supervisedEmitter_1.default([
            patternHandler('hello/**', ({ data }) => {
                calls[0]++;
                return data;
            }),
            ({ data }) => {
                calls[1]++;
                chai_1.expect(data).to.be.eql('test');
            },
        ]);
        yield SE.publish('hello/world', 'test');
        yield SE.publish('fun/world', 'test');
        chai_1.expect(calls[0]).to.be.eql(1);
        chai_1.expect(calls[1]).to.be.eql(2);
    }));
});
//# sourceMappingURL=patternHandler.spec.js.map